@@ -1,23 +1,25 @@
-
 import sys
 import posixpath
-from devpi_common.types import cached_property, ensure_unicode
+from devpi_common.types import cached_property, ensure_unicode, parse_hash_spec
 from requests.models import parse_url
 
 if sys.version_info >= (3, 0):
-    from urllib.parse import urlparse, urlunsplit, urljoin
+    from urllib.parse import urlparse, urlunsplit, urljoin, unquote
 else:
     from urlparse import urlparse, urlunsplit, urljoin
+    from urllib import unquote
+
 
 def _joinpath(url, args, asdir=False):
     new = url
     for arg in args[:-1]:
-        new = urljoin(new, arg) + "/"
+        new = urljoin(new, arg.rstrip("/")) + "/"
     new = urljoin(new, args[-1])
     if asdir:
         new = new.rstrip("/") + "/"
     return new
 
+
 class URL:
     def __init__(self, url="", *args, **kwargs):
         if isinstance(url, URL):
@@ -42,11 +44,30 @@ class URL:
     def __eq__(self, other):
         return self.url == getattr(other, "url", other)
 
+    def __ne__(self, other):
+        return not (self == other)
+
     def geturl_nofragment(self):
         """ return url without fragment """
         scheme, netloc, url, params, query, ofragment = self._parsed
         return URL(urlunsplit((scheme, netloc, url, query, "")))
 
+    @property
+    def hash_spec(self):
+        hashalgo, hash_value = parse_hash_spec(self._parsed[-1])
+        if hashalgo:
+            hashtype = self._parsed[-1].split("=")[0]
+            return "%s=%s" %(hashtype, hash_value)
+        return ""
+
+    @property
+    def hash_algo(self):
+        return parse_hash_spec(self._parsed[-1])[0]
+
+    @property
+    def hash_value(self):
+        return parse_hash_spec(self._parsed[-1])[1]
+
     def replace(self, **kwargs):
         _parsed = self._parsed
         url = []
@@ -58,6 +79,22 @@ class URL:
     def netloc(self):
         return self._parsed.netloc
 
+    @property
+    def username(self):
+        return self._parsed.username
+
+    @property
+    def password(self):
+        return self._parsed.password
+
+    @property
+    def hostname(self):
+        return self._parsed.hostname
+
+    @property
+    def port(self):
+        return self._parsed.port
+
     @property
     def scheme(self):
         return self._parsed.scheme
@@ -86,11 +123,11 @@ class URL:
 
     @property
     def basename(self):
-        return posixpath.basename(self._parsed.path)
+        return posixpath.basename(unquote(self._parsed.path))
 
     @property
     def parentbasename(self):
-        return posixpath.basename(posixpath.dirname(self._parsed.path))
+        return posixpath.basename(posixpath.dirname(unquote(self._parsed.path)))
 
     @property
     def eggfragment(self):
@@ -104,6 +141,12 @@ class URL:
         if val.startswith("md5="):
             return ensure_unicode(val[4:])
 
+    @property
+    def sha256(self):
+        val = self._parsed.fragment
+        if val.startswith("sha256="):
+            return ensure_unicode(val[4:])
+
     def joinpath(self, *args, **kwargs):
         newurl = _joinpath(self.url, args, **kwargs)
         return URL(newurl)
@@ -152,4 +195,3 @@ class URL:
         """ return url from canonical relative path. """
         scheme, netlocpath = relpath.split("/", 1)
         return cls(scheme + "://" + netlocpath)
-
