@@ -1,7 +1,9 @@
 from __future__ import unicode_literals
-
+import hashlib
+import posixpath
 import pytest
-from devpi_common.url import *
+from devpi_common.url import URL
+
 
 class TestURL:
     def test_basename(self):
@@ -30,6 +32,22 @@ class TestURL:
     def test_md5(self):
         url = URL("http://a/py.tar.gz#md5=123123")
         assert url.md5 == "123123"
+        assert url.hash_algo == hashlib.md5
+        assert url.hash_value == "123123"
+
+    @pytest.mark.parametrize("hashtype,hash_value", [
+        ("sha256", "090123"),
+        ("sha224", "1209380123"),
+        ("md5", "102938")
+    ])
+    def test_hashtypes(self, hashtype, hash_value):
+        link = URL('py-1.4.12.zip#%s=%s' % (hashtype, hash_value))
+        assert link.hash_algo == getattr(hashlib, hashtype)
+        assert link.hash_value == hash_value
+
+    def test_nohashtypes(self):
+        link = URL("whateveer#lqk=123")
+        assert link.hash_value is None and link.hash_algo is None
 
     @pytest.mark.parametrize("url,path,expected", [
         ("http://root", "dir1", "http://root/dir1"),
@@ -39,6 +57,10 @@ class TestURL:
         ("http://root/dir1/", "dir2/", "http://root/dir1/dir2/"),
         ("http://root/dir1/", "/dir2", "http://root/dir2"),
         ("http://root/dir1/", "/dir2/", "http://root/dir2/"),
+        ("http://root/dir1/dir3", "dir2", "http://root/dir1/dir2"),
+        ("http://root/dir1/dir3/", "dir2/", "http://root/dir1/dir3/dir2/"),
+        ("http://root/dir1/dir3/", "/dir2", "http://root/dir2"),
+        ("http://root/dir1/dir3/", "/dir2/", "http://root/dir2/"),
     ])
     def test_joinpath(self, url, path, expected):
         d_url = URL(url)
@@ -46,11 +68,11 @@ class TestURL:
         assert url_joined == expected
         assert URL(url, path).url == expected
 
-        assert d_url.joinpath(path, "end").url == expected + "/end"
-        assert URL(url, path, "end").url == expected + "/end"
+        assert d_url.joinpath(path, "end").url == expected.rstrip('/') + "/end"
+        assert URL(url, path, "end").url == expected.rstrip('/') + "/end"
 
-        assert d_url.joinpath(path, "end", asdir=1).url == expected + "/end/"
-        assert URL(url, path, "end", asdir=1).url == expected + "/end/"
+        assert d_url.joinpath(path, "end", asdir=1).url == expected.rstrip('/') + "/end/"
+        assert URL(url, path, "end", asdir=1).url == expected.rstrip('/') + "/end/"
 
     def test_addpath(self):
         url = URL("http://root.com/path")
@@ -70,7 +92,6 @@ class TestURL:
         url = URL(None)
         assert url.url == ""
 
-
     def test_asdir(self):
         assert URL("http://heise.de").asdir().url == "http://heise.de/"
         assert URL("http://py.org/path").asdir().url == "http://py.org/path/"
@@ -147,6 +168,45 @@ class TestURL:
         assert new_url is not url
         assert new_url.url == url.url
 
+    def test_comparison(self):
+        base = URL('https://pypi.python.org')
+        url = URL('https://pypi.python.org/simple/foo').replace(path='')
+        assert base == url
+        assert not (base != url)
+
+    def test_username(self):
+        assert URL('http://example.com').username is None
+        assert URL('http://user@example.com').username == 'user'
+        assert URL('http://user:password@example.com').username == 'user'
+        assert URL('https://example.com:443').username is None
+        assert URL('https://user@example.com:443').username == 'user'
+        assert URL('https://user:password@example.com:443').username == 'user'
+
+    def test_password(self):
+        assert URL('http://example.com').password is None
+        assert URL('http://user@example.com').password is None
+        assert URL('http://user:password@example.com').password == 'password'
+        assert URL('https://example.com:443').password is None
+        assert URL('https://user@example.com:443').password is None
+        assert URL('https://user:password@example.com:443').password == 'password'
+
+    def test_hostname(self):
+        assert URL('http://example.com').hostname == 'example.com'
+        assert URL('http://user@example.com').hostname == 'example.com'
+        assert URL('http://user:password@example.com').hostname == 'example.com'
+        assert URL('https://example.com:443').hostname == 'example.com'
+        assert URL('https://user@example.com:443').hostname == 'example.com'
+        assert URL('https://user:password@example.com:443').hostname == 'example.com'
+
+    def test_port(self):
+        assert URL('http://example.com').port is None
+        assert URL('http://user@example.com').port is None
+        assert URL('http://user:password@example.com').port is None
+        assert URL('https://example.com:443').port == 443
+        assert URL('https://user@example.com:443').port == 443
+        assert URL('https://user:password@example.com:443').port == 443
+
+
 #
 # test torelpath/fromrelpath
 #
@@ -164,4 +224,3 @@ def test_canonical_url_path_mappings(url):
     assert posixpath.normpath(path) == path
     back_url = URL.fromrelpath(path)
     assert url == back_url
-
