@@ -1,10 +1,16 @@
-
-from devpi_common.validation import *
+from devpi_common.validation import normalize_name, validate_metadata
 import pytest
 
+
 def names(*args):
     return pytest.mark.parametrize("name", args)
 
+
+@names("hello-xyz", "hello_xyz", "hello.xyz", "Hello.XYZ", "Hello___XYZ")
+def test_safe_name(name):
+    assert normalize_name(name) == "hello-xyz"
+
+
 class TestValidateMetadata:
     @names("hello", "hello-xyz", "hello1-xyz", "hello_xyz")
     def test_valid_names(self, name):
